using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace RegisterMaschine
{
    public enum CodeType { UNKNOW, JUMP, WRITE, DELETE, STOP, INIT_ALPH, INIT_REG };
    public class CodeLine
    {
        #region Fields / Properties
        private CodeType _codetype;
        public CodeType CodeType
        {
            get { return _codetype; }
        }

        private string _line;
        public string Line
        {
            get { return _line; }
        }


        private string _b;
        public string B
        {
            get { return _b; }
        }

        private string _code;
        public string Code
        {
            get { return _code; }
        }

        private string[] _params;
        public string[] Params
        {
            get { return _params; }
            set { _params = value; }
        }

        private string _comment;
        public string Comment
        {
            get { return _comment; }
        }
        #endregion

        public CodeLine(string line)
        {
            try
            {
                _line = line.Trim();

                if (line.ToLower().EndsWith("=s"))
                {
                    _codetype = CodeType.STOP;
                }
                else if (line.ToLower().StartsWith("b"))
                {                    
                    int equ = line.IndexOf("=");
                    if (equ > 0)
                        _b = line.Substring(0, equ).Trim();

                    int start_c = line.IndexOf("(");
                    int end_c = line.IndexOf(")");
                    if (start_c > 0 && end_c > 0 && start_c < end_c)
                    {
                        _code = line.Substring(start_c, end_c - start_c + 1).Trim();
                        _comment = line.Substring(end_c + 1).Trim();
                        CreateParams(_code);
                    }
                    else
                        throw new Exception("CodeLine SytaxError");
                }
                else if (line.StartsWith("#A={"))
                {
                    string tmp = line.TrimStart('#').TrimStart('A').TrimStart('=').TrimStart('{').TrimEnd('}').Trim(' ');
                    _params = tmp.Split(',');
                    _codetype = CodeType.INIT_ALPH;
                }
                else if (line.ToLower().StartsWith("#r"))
                {
                    int regno = -1;
                    int equ = line.IndexOf("=");
                    
                    string reg = null;
                    if (equ > 0)
                    {
                        reg = line.Substring(0, equ).Trim();
                    }
                    if (!string.IsNullOrEmpty(reg))
                    {
                        reg = reg.TrimStart('#').TrimStart('r');
                        regno = int.Parse(reg);
                    }
                    _codetype = CodeType.INIT_REG;
                    _b = regno.ToString();
                    _code = line.Substring(equ+1);
                }
            }
            catch 
            {
                _comment = "Syntax Error";
            }            
        }

        void CreateParams(string code)
        {
            _params = code.Trim('(', ')', ' ').Split(',');
            if (_params.Length == 2)
            {
                if (_params[1] == "0")
                {
                    _codetype = CodeType.DELETE;
                }
                else
                {
                    _codetype = CodeType.WRITE;
                }
            }
            else if (_params.Length > 2)
                _codetype = CodeType.JUMP;
            else
                _codetype = CodeType.UNKNOW;
        }
    }
}
