﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RegisterMaschine;
using System.IO;

namespace RegisterMaschine
{
    public partial class MAIN : Form
    {
        RegisterKern kern;

        public MAIN()
        {
            InitializeComponent();
            UpdateCode(this, EventArgs.Empty);
            comboBox1.SelectedIndex = 0;
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AboutBox().ShowDialog();
        }

        private void UpdateCode(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
            listBox1.Items.AddRange(RegisterHelper.ConvertToRunableCode(richTextBox1.Text));
            kern = new RegisterKern(RegisterHelper.ConvertToCode(richTextBox1.Text));
            InitializeGui();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileDialog fd = new OpenFileDialog();
            fd.Filter = "Register (*.register)|*.register";
            if (fd.ShowDialog(this) == DialogResult.OK)
            {                                
                richTextBox1.Text=File.ReadAllText(fd.FileName);
                toolStripStatusLabel1.Text = fd.FileName;
                UpdateCode(this, EventArgs.Empty);
                InitializeGui();
            }
            
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileDialog fd = new SaveFileDialog();
            fd.Filter = "Register (*.register)|*.register";
            if (fd.ShowDialog(this) == DialogResult.OK)
            {
                File.WriteAllText(fd.FileName, richTextBox1.Text);
                toolStripStatusLabel1.Text = fd.FileName;                
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string FileName=string.Empty;
            if (toolStripStatusLabel1.Text != "")
            {
                File.WriteAllText(toolStripStatusLabel1.Text, richTextBox1.Text);
            }
            else
                saveAsToolStripMenuItem_Click(this, EventArgs.Empty);                           
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            RunStep();
        }

        private void InitializeGui()
        {
            try
            {
                InitializeRegisters();
                StringBuilder sb = new StringBuilder();
                sb.Append("A={ ");
                foreach (string s in kern.Alphabet)
                {
                    sb.Append(s).Append(" , ");
                }
                labelAlph.Text = sb.ToString().Trim().TrimEnd(',') + "}";
            }
            catch 
            {
            }
        }
        private void InitializeRegisters()
        {
            registerItem1.RValue = kern.Registers[0];
            registerItem2.RValue = kern.Registers[1];
            registerItem3.RValue = kern.Registers[2];
            registerItem4.RValue = kern.Registers[3];
            registerItem5.RValue = kern.Registers[4];
            registerItem6.RValue = kern.Registers[5];
            registerItem7.RValue = kern.Registers[6];
            registerItem8.RValue = kern.Registers[7];
        }

        private void buttonRun_Click(object sender, EventArgs e)
        {
            if (comboBox1.SelectedIndex > 0)
            {
                timer1.Interval = int.Parse((string)comboBox1.SelectedItem);
                timer1.Start();
            }
            else
            {
                RunStep();
            }
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            UpdateCode(sender, e);
            
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            timer1.Stop();
        }

        private void RunStep()
        {
            try
            {
                kern.Run();

                int codepos = -1;
                for (int i = 0; i < listBox1.Items.Count; i++)
                {
                    string line = listBox1.Items[i].ToString();
                    if (line.StartsWith("->"))
                    {
                        codepos = i;
                        listBox1.Items[i] = line.Substring(2);
                    }
                }
                if (listBox1.Items.Count > codepos + 1)
                {
                    labelCommand.Text = "" + listBox1.Items[kern.CurrentCodeLineNumber];
                    listBox1.Items[kern.CurrentCodeLineNumber] = "->" + listBox1.Items[kern.CurrentCodeLineNumber];

                }
                InitializeRegisters();
            }
            catch (StopException)
            {
                timer1.Stop();
            }
            catch (Exception)
            {
                MessageBox.Show("Fehler bei der Ausführung");
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void homepageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.darkleo.com/darkleo/download.htm");
        }
    }
}