using System;
using System.Collections.Generic;
using System.Text;

namespace RegisterMaschine
{
    public class RegisterKern
    {
        private string[] alphabet;
        public string[] Alphabet
        {
            get { return alphabet; }
            set { alphabet = value; }
        }

        private string[] registers = new string[10];
        public string[] Registers
        {
            get { return registers; }
            set { registers = value; }
        }

        private List<CodeLine> codelines = new List<CodeLine>();
        public List<CodeLine> CodeLines
        {
            get { return codelines; }
            set { codelines = value; }
        }
        
        private List<CodeLine> runCodelines = new List<CodeLine>();
        public List<CodeLine> RunCodeLines
        {
            get { return runCodelines; }
            set { runCodelines = value; }
        }

        private int currentCodeLineNumber;
        public int CurrentCodeLineNumber
        {
            get { return currentCodeLineNumber; }
            set { currentCodeLineNumber = value; }
        }

        public CodeLine CurrentCodeLine
        {
            get { return runCodelines[currentCodeLineNumber]; }
        }

        public RegisterKern(string code)
        {
            string[] lines = code.Split(new char[] { '\r', '\n' });
            CodeLines.Clear();
            RunCodeLines.Clear();
            
            foreach (string line in lines)
            {
                CodeLine l = new CodeLine(line);
                if (l.CodeType != CodeType.UNKNOW)
                {
                    if (l.CodeType != CodeType.INIT_REG && l.CodeType != CodeType.INIT_ALPH)
                    {
                        RunCodeLines.Add(l);
                    }
                    else if (l.CodeType == CodeType.INIT_ALPH)
                    {
                        alphabet = l.Params;
                    }
                    CodeLines.Add(l);
                }
            }
            InitializeRegisters();
        }
        public RegisterKern(string[] lines)
        {
            CodeLines.Clear();
            RunCodeLines.Clear();
            foreach (string line in lines)
            {
                CodeLine l = new CodeLine(line);
                if (l.CodeType != CodeType.UNKNOW)
                {
                    if (l.CodeType != CodeType.INIT_REG && l.CodeType != CodeType.INIT_ALPH)
                    {
                        RunCodeLines.Add(l);
                    }
                    else if (l.CodeType == CodeType.INIT_ALPH)
                    {
                        alphabet = l.Params;
                    }
                    CodeLines.Add(l);
                }
            }
            InitializeRegisters();
        }

        public void Run()
        {
            switch (CurrentCodeLine.CodeType)
            {
                case CodeType.STOP:
                    throw new StopException();

                case CodeType.JUMP:

                    for(int i = 0;i<Alphabet.Length;i++){
                        if(Registers[int.Parse(CurrentCodeLine.Params[0])].StartsWith(alphabet[i])){
                            currentCodeLineNumber = int.Parse(CurrentCodeLine.Params[i+1]);
                            break;
                        }
                    }
                    break;

                case CodeType.WRITE:
                    string temp = alphabet[int.Parse(CurrentCodeLine.Params[1])];
                    Registers[int.Parse(CurrentCodeLine.Params[0])] = temp + Registers[int.Parse(CurrentCodeLine.Params[0])];
                    currentCodeLineNumber++;
                    break;

                case CodeType.DELETE:
                    Registers[int.Parse(CurrentCodeLine.Params[0])] = Registers[int.Parse(CurrentCodeLine.Params[0])].Substring(1);
                        currentCodeLineNumber++;
                    break;

                case CodeType.UNKNOW:
                    break;
            }
        }

        public void InitializeRegisters()
        {
            for (int i = 0; i < registers.Length; i++)
            {
                registers[i] = "@";
            }
            foreach (CodeLine line in codelines)
            {
                if (line.CodeType == CodeType.INIT_REG)
                    registers[int.Parse(line.B)] = line.Code+registers[int.Parse(line.B)];
            }
        }
    }
}
